#------------------------------------------------------------------------------
# Notes: This method adds a disk to a RHEV VM
#
#------------------------------------------------------------------------------
require 'restclient'
require 'nokogiri'

prov = $evm.root['miq_provision']
@debug = prov.options[:debug]

begin
#------------------------------------------------------------------------------
def call_rhev(servername, username, password, action, ref=nil, body_type=:xml, body=nil)
	
  	#
	# If ref is a url then use that one instead
	#
	unless ref.nil?
		url = ref if ref.include?('http')
	end
	
	url ||= "https://#{servername}#{ref}"
	
	if @debug
		$evm.log(:info, "PROV_LOG_DEBUG- Calling RHEVM at: #{url}")
		$evm.log(:info, "PROV_LOG_DEBUG- Action: #{action}")
	end
	
  	rest = RestClient::Request.new(
          :method => action,
          :url => url,
          :user => username,
          :password => password,
          :headers => { :content_type=> :xml, :accept=>:xml },
    	  :verify_ssl => false,
          :payload => body)      
  
  	response_str = rest.execute
  
  	response_str.gsub! '\"', '"'
  	response_str.gsub! '\n', ''

  	$evm.log(:info, "PROV_LOG_DEBUG- xml - #{response_str}") if @debug
    
  	response = Nokogiri::XML(response_str)
  
	return response

rescue RestClient::Exception => err
	$evm.log(:info, "The REST request failed with code: #{err.response.code}") unless err.response.nil?
	$evm.log(:info, "The response body was:\n#{err.response.body.inspect}") unless err.response.nil?
  	$evm.root['ae_result'] = 'error'
  	$evm.root['ae_reason'] = "Caught exception in add_disk:#{err.class}:#{err}"
  	$evm.log('error', "#{$evm.root['ae_reason']}")
rescue => err
	$evm.root['ae_result'] = 'error'
  	$evm.root['ae_reason'] = "Caught exception in add_disk-call_rhev:#{err.class}:#{err}"
  	$evm.log('error', "#{$evm.root['ae_reason']}")
end
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
# Start of main code
#
disk_size = prov.options[:app_disk_size]
storage_id = prov.options[:app_disk_vol]

$evm.log(:info, "PROV_LOG_DEBUG- APP Storage ID: #{storage_id}") if @debug

vm = prov.destination

storage_domain_id=nil

#
# Extract the RHEV-specific Storage Domain ID
#
unless storage_id.nil? || storage_id.blank?
	storage = $evm.vmdb('storage').find_by_id(storage_id)
	storage_domain_id = storage.ems_ref.match(/.*\/(\w.*)$/)[1]
	if @debug
		$evm.log(:info, "PROV_LOG_DEBUG- Found Storage: #{storage.name}")
		$evm.log(:info, "PROV_LOG_DEBUG- ID: #{storage.id}")
		$evm.log(:info, "PROV_LOG_DEBUG- ems_ref: #{storage.ems_ref}")
		$evm.log(:info, "PROV_LOG_DEBUG- storage_domain_id: #{storage_domain_id}")
   	end
end  

unless storage_domain_id.nil?
  #
  # Extract the IP address and credentials for the RHEV Provider
  #
  ems = vm.ext_management_system

  servername = ems.hostname
  username = ems.authentication_userid
  password = ems.authentication_password
  disk_size_bytes = (disk_size.to_i) * 1024**3
  #
  # build xml body for the RHEV REST API call
  #
  body = "<disk>"
  body += "<name>#{storage.name}</name>"
  body += "<alias>#{storage.name}</alias>"
  body += "<storage_domains>"
  body += "<storage_domain id='#{storage_domain_id}'/>"
  body += "</storage_domains>"
  body += "<size>#{disk_size_bytes}</size>"
  body += "<type>system</type>"
  body += "<interface>virtio</interface>"
  body += "<format>raw</format>"
  body += "<bootable>false</bootable>"
  body += "</disk>"

  $evm.log(:info, "PROV_LOG- Adding #{disk_size}GB disk to VM: #{vm.name}")

  if @debug
    $evm.log(:info, "PROV_LOG_DEBUG- servername: #{servername}")
    $evm.log(:info, "PROV_LOG_DEBUG- username: #{username}")
    $evm.log(:info, "PROV_LOG_DEBUG- password: #{password}")
    $evm.log(:info, "PROV_LOG_DEBUG- size: #{disk_size_bytes}")
  end

  doc = call_rhev(servername, username, password, :post, "#{vm.ems_ref}/disks", :xml, body)

  #
  # Pull out some re-usable href's from the initial response
  #
  creation_status = "complete"
  activate_href = nil
  creation_status_href = nil
  disk_href = nil

  creation_status = doc.xpath("disk/creation_status/state").text
  creation_status_href = doc.xpath("disk/link[@rel='creation_status']/@href").text
  activate_href = doc.xpath("disk/actions/link[@rel='activate']/@href").text
  disk_href =  doc.xpath("disk/@href").text

  if @debug
    $evm.log(:info, "PROV_LOG_DEBUG- Creation Status: #{creation_status}")
    $evm.log(:info, "PROV_LOG_DEBUG- creation_status_href: #{creation_status_href}")
    $evm.log(:info, "PROV_LOG_DEBUG- activate_href: #{activate_href}")
    $evm.log(:info, "PROV_LOG_DEBUG- disk_href: #{disk_href}")
  end

  #
  # Validate the creation_status (wait for up to a minute)
  #
  counter = 13
  while creation_status != "complete"
      counter -= 1
      if counter == 0
          $evm.log(:info, "PROV_LOG- Timeout waiting for new disk creation_status to reach \"complete\": Creation Status = #{creation_status}")
          break  
      else
          sleep 5
          response = call_rhev(servername, username, password, :get, creation_status_href, :xml, nil)
          creation_status = response.xpath("creation/status/state").text
        $evm.log(:info, "PROV_LOG_DEBUG- Disk Creation Status: #{creation_status} - retry: #{counter}") if @debug
      end
  end

  ##Disk has been created successfully, now check its activation status and if necessary activate it

  response = call_rhev(servername, username, password, :get, disk_href, :xml, nil)
  active_status = response.xpath("disk/active").text
  if active_status != "true"
      $evm.log(:info, "PROV_LOG_DEBUG- Activating disk") if @debug
      body = "<action/>"
      response = call_rhev(servername, username, password, :post, activate_href, :xml, body) 
  else
     $evm.log(:info, "PROV_LOG- New disk added successfully and is active")
  end
  
end
#
# Exit method
#
$evm.root['ae_result'] = 'ok'
exit MIQ_OK
#
# Set Ruby rescue behavior
#
rescue => err
	$evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = "Failed to add additional app disk:#{err.class}:#{err}"
  	$evm.log('error', "#{$evm.root['ae_reason']}")
  	#exit MIQ_ABORT
end
