#
# Description: <Method description here>
#
begin
  require 'fileutils'
  require 'git'
  require 'timeout'
  require 'yaml'

  DEFAULT_TIMEOUT = 120

  class GitLogger
    def method_missing(m, *args, &block)
      $evm.log('info', "PROV_LOG- GIT: #{args}")
    end
  end
  
  def process_yaml_updates(file_path, updates)
    $evm.log('info', "PROV_LOG- Processing YAML: [#{file_path}]")
    FileUtils.makedirs(File.dirname(file_path))
    contents = {}
    begin
      contents = YAML::load(File.open(file_path, 'r+'))
    rescue Errno::ENOENT
    end
    original_contents = Marshal.load(Marshal.dump(contents))

    (updates || {}).each do | path, update |
      parent = nil
      child = contents
      child_key = nil

      path.split(/(?<!\\)\//).each do | key |
        key_gsubbed = key.gsub(/\\/,'')
        parent = child
        child = (child[key_gsubbed] ||= {})
        child_key = key_gsubbed
      end
      value = update[:value]
      case update[:action]
      when :assign then parent[child_key] = value
      when :delete then parent.delete(child_key)
      end
    end

    updated = contents != original_contents
    File.write(file_path, contents.to_yaml) if updated
    return contents, updated
  end

  def update_puppet_git(git_repo_remote, git_repo_local, puppet_updates, hostname)
    if !puppet_updates.is_a?(Hash) or puppet_updates.empty?
      $evm.log('info', 'PROV_LOG- No updates provided.  Skipping Git commit.')
      exit MIQ_OK
    end

    begin
      $evm.log('info', 'PROV_LOG- Attempting to open hiera repo.')
      git = Git.open(git_repo_local, :log=>GitLogger.new)
    rescue ArgumentError
      $evm.log('warn', 'PROV_LOG- Unable to open hiera repo.  Cloning from source.')
      FileUtils.rm_rf(git_repo_local)
      git = Git.clone(git_repo_remote, git_repo_local)
      git.config('user.name', 'CloudForms')
      git.config('user.email', 'cloudforms@unionbank.com')
      git = Git.open(git_repo_local, :log=>GitLogger.new)
    end

    git.reset_hard('origin/HEAD')
    git.checkout('master')
    git.pull()

    hiera_results = {}
    commitmsg = ""
    git_updated = false
    git_base_path = git.dir.to_s()
    puppet_updates.each do | wildcard_file_path, updates |
      wildcard_file_paths = [wildcard_file_path]
      wildcard_file_paths = Dir.glob(File.join(git_base_path, wildcard_file_path)).collect do | file_path |
        file_path.gsub(/^#{Regexp.escape(File.join(git_base_path, ''))}/, '')
      end if wildcard_file_path.include?('*')
      $evm.log('info', "PROV_LOG- Git file path: [#{wildcard_file_paths}]")
      wildcard_file_paths.each do | file_path |
        full_file_path = File.join(git_base_path, file_path)
        hiera_results[file_path], file_updated = process_yaml_updates(full_file_path, updates)
        if file_updated
          git.add(file_path)
          commitmsg += '; ' + "#{file_path}"
          git_updated = true
        end
      end
    end

    if git_updated
      git_diff = git.diff()
      git_diff.instance_variable_set(:@to, nil)
      $evm.log('info', "PROV_LOG- Pending Hiera Git updates:\n#{git_diff}")
      #user details
	  userid =$evm.root['user'].userid.split("@").first
	  username =$evm.root['user'].name
      user = "#{userid}(#{username})" || ""
      commitmsg = "CloudForms update for #{hostname} on behalf of #{user}" + "#{commitmsg}"
      #git.commit('CloudForms update', :author=>'CloudForms <cloudforms@unionbank.com>')
      git.commit("#{commitmsg}", :author=>'CloudForms <CloudForms@unionbank.com>')
      
      git.push()
    else
      $evm.log('info', "PROV_LOG- No Hiera Git updates")
    end
    return hiera_results
  end

##############################################################################################
## MAIN ######################################################################################
##############################################################################################
  ## Set the provision and vm objects
  prov = $evm.root['miq_provision']
  @debug = prov.options[:debug]
  hostname = prov.options[:vm_name] || ""
 

  ## Local variables
  git_repo_hostname = $evm.object['git_repo_hostname']
  git_repo_path     = $evm.object['git_repo_path']
  git_repo_username = $evm.object['git_repo_username']
  git_repo_password = $evm.object.decrypt('git_repo_password')
  git_repo_remote = File.join("http://#{git_repo_username}:#{git_repo_password}@#{git_repo_hostname}", git_repo_path)
  git_repo_local = "/root/union_bank/git/hiera"  

  hiera_actions     = $evm.object['hiera_actions'] || {}

  missing_fields = ['git_repo_hostname', 
                    'git_repo_path', 
                    'git_repo_username', 
                    'git_repo_password'].collect do | key |
    key unless eval(key)
  end.compact
  unless missing_fields.empty?
    raise("Missing required variables:[#{missing_fields}]")
  end

  begin
    Timeout.timeout(DEFAULT_TIMEOUT) do
      hiera_results = update_puppet_git(git_repo_remote, git_repo_local, hiera_actions, hostname)
      $evm.log('info', "PROV_LOG- Puppet Hiera repo update results: [#{hiera_results.to_s[0..1023]}]")
      $evm.object['hiera_results'] = hiera_results
    end
  rescue Timeout::Error => e
    raise("Timed out waiting for Git:[#{e}]")
  rescue Psych::SyntaxError => e
    raise("Invalid YAML files in Hiera repo:[#{e}]")
  rescue => e
    raise("Error occurred accessing Hiera repo:[#{e}]")
  end  
  exit MIQ_OK

rescue => err
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = "Caught exception in Puppet/stashRepos/hiera_update_repo:#{err.class}:#{err}"
  $evm.log('error', "#{$evm.root['ae_reason']}")
end
